# TorVault

*TorVault* is a tool for using `OfflineMasterKey 1` in combination with a Yubikey.
It makes it possible to store long-term identity keys on a PIV slot and generate medium-term signing key and certificates from it.

## Usage

```sh
$ go build -o torvault cmd/torvault/*.go
$ ./torvault
1. Yubico YubiKey OTP+FIDO+CCID
choose a smart card: 1
choose a slot [9c]:
choose an expiration date [2025-05-24]:
choose a path for the identity public key [ed25519_master_id_public_key]:
choose a path for the signing secret key [ed25519_signing_secret_key]:
choose a path for the signing cert [ed25519_signing_cert]:

================================
smart card:			Yubico YubiKey OTP+FIDO+CCID 31892185
slot:				0x9c
public key:			EVLS6NL8wastoApvJUPDzYxJ7FAdOx22RFpqtlzFGpA
expiration date:		Sat, 24 May 2025 00:00:00 UTC (1748044800)
identity public key path:	ed25519_master_id_public_key
signing secret key path:	ed25519_signing_secret_key
signing cert path:		ed25519_signing_cert
================================

is this correct? (y/n): y
enter PIN:
success! 🧅
$ systemctl stop tor
$ cp ed25519_* /path/to/your/relay/keys
$ systemctl start tor
```

## Generate a key

*TorVault* does not support the generation of an *Ed25519* key.
This section will explain you to do that with *ykman*.

### Generate key externally (recommended)

This approach generates the key on your computer and lets you import it onto the Yubikey.
It is the recommended approach because you can make backups of it, which may be useful in the unlikely case that this software stops working.

Because this gives you access to the original private key, it is recommended to run this on an offline live system.

To generate a private key and import it onto slot `9c`:
```sh
$ openssl genpkey -algorithm ed25519 -out private.pem
# Now you can backup private.pem
$ ykman piv keys import --pin-policy ALWAYS --touch-policy ALWAYS 9c private.pem
```

### Generate key internally

This approach generates the key on the device.
The advantage of this approach is the assertion that the key has never and will never leave the device.
However, the downside is that it is impossible to have a backup of the key.

To generate a private key on slot `9c`:
```sh
$ ykman piv keys generate -a ED25519 --pin-policy ALWAYS --touch-policy ALWAYS 9c public.pem
```