package main

import (
	"bytes"
	"encoding/binary"
	"time"

	ed25519_tor "github.com/cretz/bine/torutil/ed25519"
)

// An Ed25519 Tor certificate with the `signed-with-ed25519` extension.
// This is not a general purpose implementation, rather it is tailored to our purpose.
type TorCert struct {
	ExpirationDate time.Time
	SigningKey     ed25519_tor.PublicKey
	IdentityKey    ed25519_tor.PublicKey
	Signature      [64]byte
}

// Encodes the certificate into a byte array
func (c TorCert) Encode() []byte {
	// It is HOURS since the epoch
	unix64 := c.ExpirationDate.Unix()
	unix64 = unix64 / 60 / 60

	var unix uint32 = uint32(unix64)

	res := bytes.NewBuffer([]byte{})
	res.WriteByte(0x01)                             // VERSION
	res.WriteByte(0x04)                             // CERT_TYPE
	binary.Write(res, binary.BigEndian, unix)       // EXPIRATION_DATE
	res.WriteByte(0x01)                             // CERT_KEY_TYPE
	res.Write(c.SigningKey)                         // CERTIFIED_KEY
	res.WriteByte(0x01)                             // N_EXTENSIONS
	binary.Write(res, binary.BigEndian, uint16(32)) // ExtLen
	res.WriteByte(0x04)                             // ExtType
	res.WriteByte(0x00)                             // ExtFlags
	res.Write(c.IdentityKey)                        // ExtData
	res.Write(c.Signature[:])                       // SIGNATURE

	return res.Bytes()
}

// Encodes the certificate without the signature (useful for signing)
func (c TorCert) EncodeNoSignature() []byte {
	tmp := c.Encode()
	return tmp[:len(tmp)-64]
}
